{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.neovim = {
    commentary.enable = mkEnableOption "commentary";
    completion.enable = mkEnableOption "completion";
    enable = mkEnableOption "neovim";
    formatting = {
      enable = mkEnableOption "formatting";
      formatters = mkOption {
        default = [ ];
        description = "List of formatters.";
        type = types.listOf (types.enum [
          "beautysh"
          "clang_format"
          "nixfmt"
          "prettier"
          "rustfmt"
          "yapf"
        ]);
      };
    };
    fugitive.enable = mkEnableOption "fugitive";
    harpoon.enable = mkEnableOption "harpoon";
    lsp = {
      enable = mkEnableOption "lsp";
      servers = mkOption {
        default = [ ];
        description = "List of language servers.";
        type = types.listOf (types.enum [
          "bash"
          "clangd"
          "erlang"
          "jedi"
          "nil"
          "ruff"
          "rust-analyzer"
          "typescript"
        ]);
      };
    };
    markdown-preview.enable = mkEnableOption "markdown-preview";
    snippets.enable = mkEnableOption "snippets";
    telescope.enable = mkEnableOption "telescope";
    treeSitter = {
      enable = mkEnableOption "treesitter";
      languages = mkOption {
        default = [ ];
        description = "List of languages to enable treesitter for.";
        type = types.listOf types.str;
      };
    };
    undodir = mkOption {
      default = "\${HOME}/Documents/.undo/";
      description = "Directory to store undo files in.";
      type = types.str;
    };
    undotree.enable = mkEnableOption "undotree";
    zenmode.enable = mkEnableOption "zenmode";
  };
  config = let
    cfg = config.lwad.applications.neovim;
    hasFormatter = (formatter: statement:
      optional (builtins.elem formatter cfg.formatting.formatters) statement);
    hasLsp = (server: statement:
      optional (builtins.elem server cfg.lsp.servers) statement);
  in mkIf cfg.enable {
    environment = {
      systemPackages = with pkgs;
        flatten [
          (optionals cfg.formatting.enable [
            (hasFormatter "beautysh" beautysh)
            (hasFormatter "clang_format" libclang)
            (hasFormatter "nixfmt" nixfmt)
            (hasFormatter "prettier" nodePackages.prettier)
            (hasFormatter "rustfmt" rustfmt)
            (hasFormatter "yapf" yapf)
          ])
          (optionals cfg.lsp.enable [
            (hasLsp "bash" nodePackages.bash-language-server)
            (hasLsp "clangd" libclang)
            (hasLsp "erlang" erlang-ls)
            (hasLsp "jedi" python310Packages.jedi-language-server)
            (hasLsp "nil" nil)
            (hasLsp "ruff" ruff-lsp)
            (hasLsp "rust-analyzer" rust-analyzer)
            (hasLsp "typescript" nodePackages.typescript-language-server)
          ])
          (optional cfg.telescope.enable ripgrep)
        ];
      sessionVariables.NVIM_UNDODIR = mkIf cfg.enable cfg.undodir;
      # This clangd binary must be used instead of the one in pkgs.libclang.
      variables.PATH = mkIf (builtins.elem "clangd" cfg.lsp.servers)
        "${pkgs.clang-tools}/bin:$PATH";
    };
    programs.neovim = mkIf cfg.enable {
      configure = {
        customRC = concatLines (flatten [
          "lua << EOF"
          (fileContents ./aesthetic.lua)
          (fileContents ./defaults.lua)
          (fileContents ./keymaps.lua)
          (optional cfg.commentary.enable (fileContents ./commentary.lua))
          (optional cfg.completion.enable (fileContents ./completion.lua))
          (optional cfg.harpoon.enable (fileContents ./harpoon.lua))
          (optional cfg.telescope.enable (fileContents ./telescope.lua))
          (optional cfg.treeSitter.enable (fileContents ./treesitter.lua))
          (optional cfg.undotree.enable (fileContents ./undotree.lua))
          (optional cfg.zenmode.enable
            ''vim.keymap.set("n", "<leader>z", ":ZenMode<enter>", {})'')
          (optionals
            (cfg.formatting.enable && (length cfg.formatting.formatters != 0)) [
              ''local conform = require("conform")''

              (hasFormatter "beautysh" ''
                conform.formatters.beautysh = {
                  prepend_args = { "-i", "2" },
                }
              '')

              ''
                conform.setup({
                  formatters_by_ft = {
              ''

              (hasFormatter "beautysh" ''
                bash = { "beautysh" },
                sh = { "beautysh" },
              '')
              (hasFormatter "clang_format" ''
                c = { "clang_format" },
                cpp = { "clang_format" },
                csharp = { "clang_format" },
                java = { "clang_format" },
              '')
              (hasFormatter "nixfmt" ''
                nix = { "nixfmt" },
              '')
              (hasFormatter "prettier" ''
                css = { "prettier" },
                html = { "prettier" },
                javascript = { "prettier" },
                json = { "prettier" },
                markdown = { "prettier" },
                typescript = { "prettier" },
                yaml = { "prettier" },
              '')
              (hasFormatter "rustfmt" ''
                rust = { "rustfmt" },
              '')
              (hasFormatter "yapf" ''
                python = { "yapf" },
              '')

              ''
                  },
                })
              ''

              (fileContents ./formatting.lua)
            ])
          (optionals (cfg.lsp.enable && (length cfg.lsp.servers != 0)) [
            ''local lspconfig = require("lspconfig")''

            (hasLsp "bash" "lspconfig.bashls.setup({})")
            (hasLsp "clangd" "lspconfig.clangd.setup({})")
            (hasLsp "erlang" "lspconfig.erlangls.setup({})")
            (hasLsp "jedi" "lspconfig.jedi_language_server.setup({})")
            (hasLsp "nil" "lspconfig.nil_ls.setup({})")
            (hasLsp "ruff" "lspconfig.ruff_lsp.setup({})")
            (hasLsp "rust-analyzer" "lspconfig.rust_analyzer.setup({})")
            (hasLsp "typescript" "lspconfig.tsserver.setup({})")

            (fileContents ./lsp.lua)
          ])
          "EOF"
        ]);
        packages.myVimPackage.start = with pkgs.vimPlugins;
          flatten [
            (hasFormatter "prettier" vim-prettier)
            (nvim-treesitter.withPlugins (plugins:
              map (language: getAttr language plugins)
              (optionals cfg.treeSitter.enable cfg.treeSitter.languages)))
            (optional cfg.commentary.enable vim-commentary)
            (optional cfg.formatting.enable conform-nvim)
            (optional cfg.fugitive.enable vim-fugitive)
            (optional cfg.harpoon.enable harpoon)
            (optional cfg.lsp.enable nvim-lspconfig)
            (optional cfg.markdown-preview.enable markdown-preview-nvim)
            (optional cfg.snippets.enable luasnip)
            (optional cfg.telescope.enable telescope-nvim)
            (optional cfg.undotree.enable undotree)
            (optional cfg.zenmode.enable zen-mode-nvim)
            (optionals cfg.completion.enable [
              (optional cfg.lsp.enable cmp-nvim-lsp)
              (optional cfg.snippets.enable cmp_luasnip)
              cmp-buffer
              cmp-cmdline
              cmp-path
              nvim-cmp
            ])
            lualine-nvim
            vim-repeat
          ];
      };
      defaultEditor = true;
      enable = true;
    };
  };
}
