{ config, inputs, lib, pkgs, ... }:

with lib; {
  options.lwad.core.enable = mkEnableOption "core config";
  config = let cfg = config.lwad.core.enable;
  in mkIf cfg {
    console = {
      keyMap = lib.mkForce "uk";
      useXkbConfig = true;
    };
    documentation = {
      doc.enable = false;
      nixos.enable = false;
    };
    environment = {
      defaultPackages = lib.mkForce [ ];
      sessionVariables = { # home decluttering
        ANDROID_HOME = "\${XDG_DATA_HOME}/android";
        ANDROID_USER_HOME = "\${XDG_DATA_HOME}/android";
        CUDA_CACHE_PATH = "\${XDG_CACHE_HOME}/nv";
        GNUPGHOME = "\${XDG_DATA_HOME}/gnupg";
        GRADLE_USER_HOME = "\${XDG_DATA_HOME}/gradle";
        GTK2_RC_FILES = "\${XDG_CONFIG_HOME}/gtk-3.0/gtkrc";
        HISTFILE = "\${XDG_STATE_HOME}/bash/history";
        IONIC_CONFIG_DIRECTORY = "\${XDG_CONFIG_HOME}/ionic";
        _JAVA_OPTIONS = "-Djava.util.prefs.userRoot=\${XDG_CONFIG_HOME}/java";
        MYSQL_HISTFILE = "\${XDG_DATA_HOME}/mysql_history";
        NODE_REPL_HISTORY = "\${XDG_DATA_HOME}/node_repl_history";
        PSQL_HISTORY = "\${XDG_DATA_HOME}/psql_history";
        XCOMPOSECACHE = "\${XDG_CACHE_HOME}/X11/xcompose";
        XDG_CACHE_HOME = "$HOME/.cache";
        XDG_CONFIG_HOME = "$HOME/.config";
        XDG_DATA_HOME = "$HOME/.local/share";
        XDG_STATE_HOME = "$HOME/.local/state";
      } // { # other
        HISTCONTROL = "ignorespace";
      };
    };
    hardware.i2c.enable = true;
    home-manager = {
      useGlobalPkgs = true;
      users.${config.lwad.definitions.username} = {
        home = {
          homeDirectory = "/home/${config.lwad.definitions.username}";
          username = "${config.lwad.definitions.username}";
        };
        programs.home-manager.enable = true;
      };
      useUserPackages = true;
    };
    i18n = {
      defaultLocale = "en_GB.UTF-8";
      extraLocaleSettings = {
        LC_ADDRESS = "en_GB.UTF-8";
        LC_IDENTIFICATION = "en_GB.UTF-8";
        LC_MEASUREMENT = "en_GB.UTF-8";
        LC_MONETARY = "en_GB.UTF-8";
        LC_NAME = "en_GB.UTF-8";
        LC_PAPER = "en_GB.UTF-8";
        LC_TELEPHONE = "en_GB.UTF-8";
      };
    };
    nix = {
      gc = {
        automatic = true;
        dates = "weekly";
        options = "--delete-older-than 7d";
      };
      nixPath =
        [ "nixpkgs=${inputs.nixpkgs}" "home-manager=${inputs.home-manager}" ];
      registry.nixpkgs.flake = inputs.nixpkgs;
      settings = {
        allowed-users = [ "${config.lwad.definitions.username}" "root" ];
        auto-optimise-store = true;
        experimental-features = [ "nix-command" "flakes" ];
        trusted-users = [ "${config.lwad.definitions.username}" "root" ];
      };
    };
    nixpkgs.config.allowUnfree = true;
    networking = {
      networkmanager.enable = true;
      nftables.enable = true;
    };
    programs.bash.shellAliases = {
      cp = "cp -i";
      n = "nvim";
      nixos-status = "journalctl -xefu nixos-upgrade.service";
      nixos-upgrade =
        "${config.lwad.definitions.sudoas.command} systemctl start nixos-upgrade.service";
      nixos-upgrade-local =
        "${config.lwad.definitions.sudoas.command} nixos-rebuild switch --no-write-lock-file --recreate-lock-file --flake /home/${config.lwad.definitions.username}/nixos#";
    };
    services = {
      fstrim = {
        enable = true;
        interval = "weekly";
      };
      xserver.xkb.options = "caps:escape";
    };
    system = {
      autoUpgrade = {
        dates = "00:00";
        enable = true;
        flags = [ "--recreate-lock-file" "--refresh" "--no-write-lock-file" ];
        flake = "git+ssh://disroot/lwad/nixos.git";
        persistent = true;
      };
    };
    systemd.services = {
      NetworkManager-wait-online.enable = lib.mkForce false;
      nixos-upgrade.preStart =
        "while ! ${pkgs.unixtools.ping}/bin/ping -c 1 -W 1 cache.nixos.org; do ${pkgs.coreutils}/bin/sleep 1; done";
      systemd-networkd-wait-online.enable = lib.mkForce false;
    };
    time.timeZone = "Europe/London";
    users = {
      defaultUserShell = pkgs.bash;
      users.${config.lwad.definitions.username} = {
        isNormalUser = true;
        extraGroups = [ "networkmanager" "wheel" ];
      };
    };
  };
}
