{ config, lib, pkgs, ... }:

{
  fileSystems."/mnt/h" = {
    device = "//homes.lancs.ac.uk/18/waddingl";
    fsType = "cifs";
    options = [
      "noauto"
      "nofail"
      "_netdev"
      "x-systemd.automount"
      "x-systemd.device-timeout=5s"
      "x-systemd.mount-timeout=5s"
      "x-systemd.idle-timeout=60s"
      "credentials=/home/${config.lwad.definitions.username}/.smbcredentials"
      "gid=users"
      "uid=${config.lwad.definitions.username}"
      "noperm"
      "noatime"
    ];
  };
  hardware.bluetooth = {
    disabledPlugins = [ "input" ];
    enable = true;
  };
  home-manager.users.${config.lwad.definitions.username} = {
    home.stateVersion = "22.11";
    programs.firefox.profiles.default.settings."network.trr.mode" =
      lib.mkForce 2;
    xsession.windowManager.i3.config.keybindings."Print" =
      "exec --no-startup-id ${
        let
          cut = "${pkgs.coreutils}/bin/cut";
          rg = "${pkgs.ripgrep}/bin/rg";
          xinput = "${pkgs.xorg.xinput}/bin/xinput";
        in pkgs.writeScript "touchpad_toggle.sh" ''
          device_id="$(${xinput} | ${pkgs.gnused}/bin/sed -e "s/[^a-zA-Z]*//" -e "s/id=//" | ${rg} "Synaptics" | ${cut} -f 2)"
          state="$(${xinput} list-props "$device_id" | ${rg} "Device Enabled" | ${cut} -f 3)"
          state=$((1 - state))
          ${xinput} set-prop "$device_id" "Device Enabled" "$state"
        ''
      }";
  };
  imports = [
    ../components/desktop-environment.nix
    ../lwad
    ./hardware-configuration.nix
  ];
  lwad = {
    applications = {
      i3 = {
        audioControls = {
          bluetooth.enable = true;
          commands = let
            mute_audio = pkgs.writeScript "mute_audio.sh" ''
              IFS=$'\n'
              sinks=$(${pactl} list short sinks | cut -f 1)
              for sink in $sinks; do
                ${pactl} set-sink-volume $sink 0
              done
            '';
            pactl = "${pkgs.pulseaudio}/bin/pactl";
          in {
            headphones = ''
              ${mute_audio} && ${pactl} set-default-sink $(${pactl} list sinks | rg "Name: " | cut -d " " -f 2 | rg "Focusrite_Scarlett_2i2") && ${pkgs.pamixer}/bin/pamixer --set-volume 0'';
            speakers = ''
              ${mute_audio} && ${pactl} set-default-sink $(${pactl} list sinks | rg "Name: " | cut -d " " -f 2 | rg "analog-stereo") && ${pkgs.pamixer}/bin/pamixer --set-volume 0'';
          };
          enable = true;
        };
        backlight.enable = true;
        displayProperties = {
          internal = {
            width = 1920;
            height = 1080;
            output = "eDP-1";
          };
          ultrawide = {
            width = 2560;
            height = 1080;
            output = "HDMI-1-.*";
          };
          panasonic = {
            width = 1280;
            height = 720;
            output = "HDMI-1-.*";
            extraArgs = ''
              --set underscan on --set "underscan hborder" 34 --set "underscan vborder" 19'';
          };
        };
        moveDisplayUp.enable = true;
        outputWorkspaceMappings = {
          "HDMI-1-1 HDMI-1-2 eDP-1" = [ 1 2 3 4 5 ];
          "eDP-1 HDMI-1-1 HDMI-1-2" = [ 6 7 8 9 10 ];
        };
      };
      terminalEmulator = {
        alacritty.fontSize = 7.0;
        kitty.fontSize = 9.0;
      };
    };
    filesystems.simple.enable = true;
    utilities.btrbk.snapshots = [{
      volume = "/";
      subvolume = "home/${config.lwad.definitions.username}/Documents";
      snapshotDir = "home/${config.lwad.definitions.username}/snapshots";
      sync = {
        enable = false;
        serverAddress = "shinobu";
        serverBackupDir = "/backups/${config.networking.hostName}";
      };
    }];
  };
  networking = {
    hostName = "ryuko";
    firewall.interfaces."tailscale0".allowedTCPPorts = [ 8081 ];
  };
  programs = {
    bash.shellAliases.rdiff =
      "rsync -nilrc --delete akame:Documents/ ~/Documents/ --exclude-from=/home/${config.lwad.definitions.username}/Documents/exclude";
    light.enable = true;
    steam.enable = true;
    tmux = {
      extraConfigBeforePlugins = ''
        set-option -g @batt_icon_status_charged " "
        set-option -g @batt_icon_status_charging " ▲"
        set-option -g @batt_icon_status_discharging " ▼"
        set-option -g @batt_icon_status_attached " "
        set-option -g @batt_icon_status_unknown " ▲"
        set-option -g status-right "#{battery_percentage}#{battery_icon_status}"
      '';
      plugins = with pkgs.tmuxPlugins; [ battery ];
    };
  };
  services = {
    logind.extraConfig = ''
      HandlePowerKey=ignore
    '';
    tlp = {
      enable = true;
      settings = {
        TLP_ENABLE = 1;
        START_CHARGE_THRESHHOLD_BAT0 = 75;
        STOP_CHARGE_THRESHHOLD_BAT0 = 85;
        DISK_DEVICES = "nvme0n1";
        DISK_APM_LEVEL_ON_AC = "254 254";
        DISK_APM_LEVEL_ON_BAT = "254 254";
        PLATFORM_PROFILE_ON_AC = "performace";
        PLATFORM_PROFILE_ON_BAT = "low-power";
        CPU_SCALING_GOVERNOR_ON_AC = "powersave";
        CPU_SCALING_GOVERNOR_ON_BAT = "powersave";
        CPU_MIN_PERF_ON_AC = 0;
        CPU_MAX_PERF_ON_AC = 200;
        CPU_MIN_PERF_ON_BAT = 0;
        CPU_MAX_PERF_ON_BAT = 30;
        CPU_BOOST_ON_AC = 1;
        CPU_BOOST_ON_BAT = 0;
      };
    };
    xserver = {
      libinput.touchpad = {
        disableWhileTyping = true;
        naturalScrolling = true;
      };
      synaptics = {
        maxSpeed = "1";
        minSpeed = "1";
        palmDetect = true;
        twoFingerScroll = true;
      };
      xrandrHeads = [{
        output = "eDP-0";
        primary = true;
        monitorConfig = ''
          Option  "Position"      "0 0"
          Option  "PreferredMode" "1920x1080"
          Option  "DPMS"          "true"
        '';
      }];
    };
  };
  system.stateVersion = "22.11";
}
